/*
 **************************************************************************
 * File Name    : wm8988_i2c.c
 * Description  : wm 8988 command driver
 * Author       : Artery SW Team
 * Date         : 2018-11-26
 * Version      : V1.0.0
 **************************************************************************
 */
 
#include "at32f403a_407.h"
#include "wm8988_i2c.h"
#include "at32f403a_407_board.h"

#define AT32_I2C_SDA_PIN        GPIO_PINS_9
#define AT32_I2C_SCL_PIN        GPIO_PINS_8
#define AT32_I2Cx               I2C1
#define AT32_I2C_SPEED          100000   /*i2c speed*/

/**
  * @brief  I2C GPIO initialize
  *         
  * @param  None
  * @retval None
  */
void AT32_I2C_Init(void)
{
  gpio_init_type  GPIO_InitStructure;

  /*i2c remap*/
  crm_periph_clock_enable(CRM_GPIOB_PERIPH_CLOCK,TRUE);
  crm_periph_clock_enable(CRM_I2C1_PERIPH_CLOCK,TRUE);
  crm_periph_clock_enable(CRM_IOMUX_PERIPH_CLOCK,TRUE);
  gpio_pin_remap_config(I2C1_MUX, TRUE);
  
  /*i2c gpio configure*/
  gpio_default_para_init(&GPIO_InitStructure);
  GPIO_InitStructure.gpio_pins = AT32_I2C_SDA_PIN | AT32_I2C_SCL_PIN;
  GPIO_InitStructure.gpio_mode = GPIO_MODE_MUX;
  GPIO_InitStructure.gpio_out_type = GPIO_OUTPUT_OPEN_DRAIN;
  GPIO_InitStructure.gpio_drive_strength =  GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_init(GPIOB, &GPIO_InitStructure);

  /*********** I2C periphral configuration **********/
  i2c_reset(AT32_I2Cx);
  
  i2c_init(AT32_I2Cx, I2C_FSMODE_DUTY_2_1, AT32_I2C_SPEED);
  i2c_own_address1_set(AT32_I2Cx, I2C_ADDRESS_MODE_7BIT, 0x00);
  
  i2c_enable(AT32_I2Cx, TRUE);
    
}

uint8_t i2c_wait_flag(i2c_type *i2c_x, uint32_t flag, flag_status status, uint32_t timeout)
{
  while(i2c_flag_get(i2c_x, flag) == status)
  {    
    /* check timeout */
    if((timeout--) == 0)
    {      
      return I2C_FAILED;
    }
  }
  
  return I2C_SUCCESS;
}

void i2c_addr_flag_clear(i2c_type *i2c_x)
{
  __IO uint32_t tmpreg;  
  
  tmpreg = i2c_x->sts1; 
  
  tmpreg = i2c_x->sts2; 
} 

/**
  * @brief  I2C Write data to reg
  *         
  * @param  None
  * @retval None
  */
uint8_t AT32_I2C_WriteReg(uint8_t u8DevAddr, uint8_t u8RegAddr, uint8_t u8RegData)
{
  delay_us(200);
  if(i2c_wait_flag(AT32_I2Cx, I2C_BUSYF_FLAG, SET, 10000) != I2C_SUCCESS )
  {
      return I2C_FAILED;
  }
  /*send the start bit*/
  i2c_start_generate(AT32_I2Cx);
  /*wait start generated*/
  if(i2c_wait_flag(AT32_I2Cx, I2C_STARTF_FLAG, RESET, 10000) != I2C_SUCCESS )
  {
      return I2C_FAILED;
  }
  
  /*send device address*/
  i2c_7bit_address_send(AT32_I2Cx, u8DevAddr, I2C_DIRECTION_TRANSMIT); 

  if(i2c_wait_flag(AT32_I2Cx, I2C_ADDR7F_FLAG, RESET, 10000) != I2C_SUCCESS )
  {
      i2c_stop_generate(AT32_I2Cx);
      return I2C_FAILED;
  }
  
  i2c_addr_flag_clear(AT32_I2Cx); 
  
  if ( i2c_wait_flag(AT32_I2Cx, I2C_TDBE_FLAG, RESET, 10000) != I2C_SUCCESS )
  {
      i2c_stop_generate(AT32_I2Cx);
      return I2C_FAILED;
  }
  
  /*send register address*/
  i2c_data_send(AT32_I2Cx, u8RegAddr); 
  
  /*wait ack*/
  if ( i2c_wait_flag(AT32_I2Cx, I2C_TDBE_FLAG, RESET, 10000) != I2C_SUCCESS )
  {
      i2c_stop_generate(AT32_I2Cx);
      return I2C_FAILED;
  }
  /*send data*/
  i2c_data_send(AT32_I2Cx, u8RegData); 
  /*wait ack*/
  if ( i2c_wait_flag(AT32_I2Cx, I2C_TDC_FLAG, RESET, 10000) != I2C_SUCCESS )
  {
      i2c_stop_generate(AT32_I2Cx);
      return I2C_FAILED;
  }
  /* Send STOP condition */ 
  i2c_stop_generate(AT32_I2Cx);
  return I2C_SUCCESS;
}

/**
  * @brief  I2C read data from reg
  *         
  * @param  None
  * @retval None
  */
uint8_t AT32_I2C_ReadReg(uint8_t u8DevAddr, uint8_t u8RegAddr, uint8_t *u8RegData)
{

    return 0;
}
